@@ -17,8 +17,6 @@ our @EXPORT = qw(
 );
 
 my %TYPE;
-my %COERCE;
-my %COERCE_KEYS;
 
 sub as ($) {
     return(as => $_[0]);
@@ -74,7 +72,7 @@ BEGIN {
         );
     }
 
-    sub optimized_constraints {
+    sub optimized_constraints { # DEPRECATED
         Carp::cluck('optimized_constraints() has been deprecated');
         return \%TYPE;
     }
@@ -186,7 +184,7 @@ sub role_type {
     );
 }
 
-sub typecast_constraints {
+sub typecast_constraints { # DEPRECATED
     my($class, $pkg, $type, $value) = @_;
     Carp::croak("wrong arguments count") unless @_ == 4;
 
